@@ -538,11 +538,11 @@ def test_name_consistent_across_instances():
     func = delayed(identity, pure=True)
 
     data = {"x": 1, "y": 25, "z": [1, 2, 3]}
-    assert func(data)._key == "identity-02129ed1acaffa7039deee80c5da547c"
+    assert func(data)._key == "identity-4f318f3c27b869239e97c3ac07f7201a"
 
     data = {"x": 1, 1: "x"}
     assert func(data)._key == func(data)._key
-    assert func(1)._key == "identity-ca2fae46a3b938016331acac1908ae45"
+    assert func(1)._key == "identity-7258833899272585e16d0ec36b21a3de"
 
 
 def test_sensitive_to_partials():
@@ -730,3 +730,13 @@ def test_annotations_survive_optimization():
     assert len(d_opt.dask.layers) == 1
     assert len(d_opt.dask.layers["b"]) == 2  # c is culled
     assert d_opt.dask.layers["b"].annotations == {"foo": "bar"}
+
+
+def test_delayed_function_attributes_forwarded():
+    @delayed
+    def add(x, y):
+        """This is a docstring"""
+        return x + y
+
+    assert add.__name__ == "add"
+    assert add.__doc__ == "This is a docstring"
